#!/usr/bin/env bash
#!/bin/env bash

## Name: configure.sh
## Version: 1.2.6
## Date: May 17, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Configure


ECHO_N="echo -n"
ECHO_E="echo -e"
ECHO_="echo"
AWK="awk"
SAMTOOLS="samtools"
GENOMECOVERAGEBED="genomeCoverageBed"
BAMTOBED="bamToBed"
PREFIX="."
MAP=""
GFF=""
CLUSTER_OPTION=""
PAIREND_OPTION=""
LIBTYPE="fr-unstranded"
REPLICATE_INTRON=0
REPLICATE_EXON=0
MAXCPU=1
MEM="2G"

usage() {
    $ECHO_E "Usage: ./configure.sh [OPTIONS]\n"
    $ECHO_E "Required:"
    $ECHO_E "-d|--description FILE"
    $ECHO_E "\tA tab-delimited file describing mapping files for experiments."
    $ECHO_E "\tFormat:"
    $ECHO_E "\tfile\texperiment"
    $ECHO_E "\t~/data/A.bam\tprimary"
    $ECHO_E "\t~/data/B.bam\tprimary"
    $ECHO_E "\t~/data/C.bam\tprimary"
    $ECHO_E "\t~/data/D.bam\tsecondary"
    $ECHO_E "\t~/data/E.bam\tsecondary"
    $ECHO_E "\t~/data/F.bam\tsecondary\n"
    $ECHO_E "Optional:"
    $ECHO_E "-c|--cluster"
    $ECHO_E "\tScripts are generated in Sun Grid Engine format. By default, shell scripts are generated."
    $ECHO_E "-C|--CPU MAXCPU"
    $ECHO_E "\tRun up to MAXCPU processes in parallel. By default, 1."
    $ECHO_E "-f|--feature FEATURE"
    $ECHO_E "\tFeatures in FEATURE will be selected. By default, gene and pseudogene, followed by their exons, are selected for analysis."
    $ECHO_E "-g|--gff GFF"
    $ECHO_E "\tGene and exon features are required in GFF annotation. If not given, GFFprof must be called to generate a GFF from RNA-seq data."
    $ECHO_E "-h|-?|--help"
    $ECHO_E "\tPrint this help."
    $ECHO_E "-i|--indexing"
    $ECHO_E "\tIndexing for visualization is created (.bai, .wig)."
    $ECHO_E "-l|--library-type LIBTYPE"
    $ECHO_E "\tThe library type should be provided to consider the correct RNA-seq protocol. The default is unstranded (fr-unstranded). Other options are fr-firststrand or fr-secondstrand."
    $ECHO_E "-m|--memory INT"
    $ECHO_E "\tThe maxium required memory for each process, specified either in bytes or with a K, M, or G suffix. By default, 2G."
    $ECHO_E "-M|--MaxInsertSize MIS"
    $ECHO_E "\tMaxium insert size is considered in case of paired-end reads. If not given, size of the longest gene will be chosen."
    $ECHO_E "-p|--prefix PREFIX"
    $ECHO_E "\tAnalysis will be done in PREFIX. By default, the current directory . is used."
    $ECHO_E "-P|--Pair"
    $ECHO_E "\tThis option should be used for paired-end reads. By default, single-end reads are considered."
    $ECHO_E "-w|--wholeDE"
    $ECHO_E "\tDifferential expression on the whole genes with htseq-count and DESeq is analyzed. By default, this analysis is not executed.\n"

    exit 1
}

if [ ! $1 ]
then
    usage
fi

while :
do
    case $1 in
	-h | --help | -\?)
	    usage
	    ;;
	-c | --cluster)
	    CLUSTER_OPTION="--cluster"
	    shift
	    ;;
	-i | --indexing)
	    INDEX_OPTION="--indexing"
	    shift
	    ;;
	-P | --Pair)
	    PAIREND_OPTION="--Pairend"
	    shift
	    ;;
	-w | --wholeDE)
	    WHOLEDE_OPTION="--wholeDE"
	    shift
	    ;;
	-C | --CPU)
	    MAXCPU=$2
	    shift 2
	    ;;
	--CPU=*)
            MAXCPU=${1#*=}
            shift
            ;;
	-p | --prefix)
	    PREFIX=$2
	    shift 2
	    ;;
	--prefix=*)
	    PREFIX=${1#*=} #delete everything up till '='
	    shift
	    ;;
	-d | --description)
	    MAP=$2
	    shift 2
	    ;;
	--description=*)
	    MAP=${1#*=}
	    shift
	    ;;
	-g | --gff)
	    GFF=$2
	    shift 2
	    ;;
	--gff=*)
            GFF=${1#*=}
            shift
            ;;
	-l | --library-type)
	    LIBTYPE=$2
	    shift 2
	    ;;
	--library-type=*)
            LIBTYPE=${1#*=}
            shift
            ;;
	-m | --memory)
	    MEM=$2
	    shift 2
	    ;;
	--memory=*)
            MEM=${1#*=}
            shift
            ;;
	-M | --MaxInsertSize)
	    MIS=$2
	    shift 2
	    ;;
	--MaxInsertSize=*)
            MIS=${1#*=}
            shift
            ;;
	-f | --feature)
	    FEATURE=$2
	    shift 2
	    ;;
	--feature=*)
            FEATURE=${1#*=}
            shift
            ;;
	-*)
	    $ECHO_ "Unknown option: $1" >&2
	    shift
	    ;;
	--) #end of all options
	    shift
	    break
	    ;;
	*) #no more options, stop while loop
	    break
	    ;;
    esac
done

$ECHO_E "\nCONFIGURATION..."

## directory containing sources
SRC=$0
SRC=`dirname $SRC`
cd $SRC
SRC=$PWD
cd - > /dev/null

## directory containing analysis
$ECHO_N "Checking for PREFIX... "
if [ ! -d "$PREFIX" ]
then
    $ECHO_ "no"
    mkdir -p $PREFIX
    $ECHO_E "\tCreated. Analysis is done in $PREFIX"
else
    $ECHO_ "yes"
    $ECHO_E "\tAnalysis is done in $PREFIX"
fi
cd $PREFIX
PREFIX=$PWD
cd - > /dev/null

## check curve directory
$ECHO_N "Checking for curve directory... "
if [ ! -d "$PREFIX/curve" ]
then
    $ECHO_ "no"
    mkdir $PREFIX/curve
    $ECHO_E "\tCreated. Coverage profiles are generated in $PREFIX/curve"
else
    $ECHO_ "yes"
    $ECHO_E "\tCoverage profiles are generated in $PREFIX/curve"
fi

## check genome directory
$ECHO_N "Checking for genome directory... "
if [ ! -d "$PREFIX/genome" ]
then
    $ECHO_ "no"
    mkdir $PREFIX/genome
    $ECHO_E "\tCreated. Analysis on given gff is done in $PREFIX/genome"
else
    $ECHO_ "yes"
    $ECHO_E "\tAnalysis on given gff is done in $PREFIX/genome"
fi

## check result directory
$ECHO_N "Checking for result directory... "
if [ ! -d "$PREFIX/result" ]
then
    $ECHO_ "no"
    mkdir $PREFIX/result
    $ECHO_E "\tCreated. RNAprof results are produced in $PREFIX/result"
else
    $ECHO_ "yes"
    $ECHO_E "\tRNAprof results are produced in $PREFIX/result"
fi

## check rnaseq directory
$ECHO_N "Checking for rnaseq directory... "
if [ ! -d "$PREFIX/rnaseq" ]
then
    $ECHO_ "no"
    mkdir $PREFIX/rnaseq
    mkdir $PREFIX/rnaseq/gff
    mkdir $PREFIX/rnaseq/exon
    mkdir $PREFIX/rnaseq/count
    if [ $WHOLEDE_OPTION ]
    then
	mkdir $PREFIX/rnaseq/htseqcount
    fi
    $ECHO_E "\tCreated. RNAseq data are generated in $PREFIX/rnaseq"
else
    $ECHO_ "yes"
    if [ -d "$PREFIX/rnaseq/gff" ]
    then
	rm -rf $PREFIX/rnaseq/gff
    fi
    mkdir $PREFIX/rnaseq/gff
	
    if [ -d "$PREFIX/rnaseq/exon" ]
    then
	rm -rf $PREFIX/rnaseq/exon
    fi
    mkdir $PREFIX/rnaseq/exon
    if [ -d "$PREFIX/rnaseq/count" ]
    then
	rm -rf $PREFIX/rnaseq/count
    fi
    mkdir $PREFIX/rnaseq/count
    if [ -d "$PREFIX/rnaseq/htseqcount" ]
    then
	rm -rf $PREFIX/rnaseq/htseqcount
    fi
    if [ $WHOLEDE_OPTION ]
    then
	mkdir $PREFIX/rnaseq/htseqcount
    fi
    $ECHO_E "\tRNAseq data are generated in $PREFIX/rnaseq"
fi

## check script directory
$ECHO_N "Checking for script directory... "
if [ ! -d "$PREFIX/script" ]
then
    $ECHO_ "no"
    mkdir $PREFIX/script
    $ECHO_E "\tCreated. Scripts are genenated in $PREFIX/script"
else
    $ECHO_ "yes"
    $ECHO_E "\tExisting scripts are deleted and new scripts will be generated in $PREFIX/script"
    rm -f $PREFIX/script/*
    # while true
    # do
    # 	read -p "Are you sure? [y/n]" yn
    # 	case $yn in
    # 	    [Yy]*) 
    # 		rm -f $PREFIX/script/*
    # 		break
    # 		;;
    # 	    [Nn]*) 
    # 		exit
    # 		;;
    # 	    *)
    # 		$ECHO_E "Please answer yes or no."
    # 		;;
    # 	esac
    # done
fi

## check awk
$ECHO_N "Checking for awk..."
if ! type $AWK > /dev/null 2>&1
then
    $ECHO_ "no"
    $ECHO_E "\tawk is required in \$PATH"
    exit 1
else
    $ECHO_ "yes"
fi

## check mapping option
$ECHO_N "Checking mapping option... "
if [ ! "$MAP" ]
then
    $ECHO_ "option '--mapping MAP' not given. See --help"
    exit 1
fi
if [ ! -e "$MAP" ]
then
    $ECHO_ "mapping file description not found"
    exit 1
fi
bamnoraw=`cat $MAP | wc -l`
if [ $bamnoraw -lt 2 ]
then
    $ECHO_ "wrong format in mapping file description"
    exit 1
fi
$ECHO_ "yes"

## check mapping files
$ECHO_N "Checking for mapping bam files... "
if [ -e "$PREFIX/rnaseq/bamlist.txt" ]
then
    rm $PREFIX/rnaseq/bamlist.txt
fi
$ECHO_E "label\tfile\texperiment" > $PREFIX/rnaseq/description.txt
#awk 'NR>1 {print $0}' $MAP > bamlist
tail -n +2 $MAP > bamlist
while read line
do
    CUR_PWD=$PWD
    MAP_PREFIX=`dirname $MAP`
    cd $MAP_PREFIX
    DESCARR=(${line//"\t"/ })
    if [ ! -e "${DESCARR[0]}" ]
    then
	$ECHO_ "no"
	exit 1
    else
	MAP_PREFIX=`dirname ${DESCARR[0]}`
	cd $MAP_PREFIX
	MAP_PREFIX=$PWD
	cd - > /dev/null
	bam=`basename ${DESCARR[0]}`
	instance=`basename $bam .bam`
	instance=`basename $instance accepted_hits`
	instance=`basename $instance .`
	instance=`basename $instance _`
	$ECHO_E "${DESCARR[1]}:$instance\t$instance\t${DESCARR[1]}" >> "$PREFIX/rnaseq/description.txt"
	bam=${MAP_PREFIX}/$bam
	$ECHO_E "$bam" >> "$PREFIX/rnaseq/bamlist.txt"
    fi
    cd $CUR_PWD
done < bamlist
$ECHO_ "yes"
bamnofin=`awk 'NR>1 {print $2}' $PREFIX/rnaseq/description.txt | uniq | wc -l`
bamnoraw=`cat bamlist | wc -l`
if [ $bamnofin -lt $bamnoraw ]
then
    $ECHO_E "Error: Bam file names should be different"
    exit 1
fi
awk '{print $2}' bamlist | sort | uniq -c | awk '{print $1}' > replicates
while read line
do
    if [ $REPLICATE_EXON -lt $line ]
    then
     	REPLICATE_EXON=$line
    fi
    REPLICATE_INTRON=`expr $REPLICATE_INTRON + $line`
done < replicates
rm bamlist
rm replicates

## check gff option
if [ -z "$GFF" ]
then
    $ECHO_ "Warning: option '--gff GFF' not given. It is required to run GFFprof to get the annotation from RNAseq data."
    $ECHO_N > $PREFIX/rnaseq/new_annotation.gff
    GFF="$PREFIX/rnaseq/new_annotation.gff"
else
    ##directory containing gff
    GFF_PREFIX=`dirname $GFF`
    cd $GFF_PREFIX
    GFF_PREFIX=$PWD
    cd - > /dev/null
    
    ##check gff file
    $ECHO_N "Checking annotated gff file... "
    GFF=`basename $GFF`
    GFF=${GFF_PREFIX}/$GFF
    if [ ! -e "$GFF" ]
    then
	$ECHO_ "no"
	$ECHO_E "\tError: Annotated GFF file not found."
	exit 1
    else
        lastfeature=`cut -f3 ${GFF} | tail -1`
        if [[ $lastfeature == "gene" ]] || [[ $lastfeature == "transcript" ]] || [[ $lastfeature == "pseudogene" ]]
	then
	    $ECHO_E "\tError: Annotated GFF file wrongly formatted."
	    #exit 1
	else
	    $ECHO_ "yes"
	fi
    fi
fi

## check feature file
$ECHO_N "Checking for feature.list... "
if [ ! -e "$FEATURE" ]
then
    $ECHO_ "no"
    $ECHO_ "gene" > $PREFIX/genome/feature.list
    $ECHO_ "pseudogene" >> $PREFIX/genome/feature.list
    $ECHO_E "\tfeature.list is created in $PREFIX/genome"
    FEATURE="$PREFIX/genome/feature.list"
else
    $ECHO_ "yes"
fi

## check samtools
$ECHO_N "Checking for samtools..."
if ! type $SAMTOOLS > /dev/null 2>&1
then
    $ECHO_ "no"
    $ECHO_E "\tsamtools is required in \$PATH"
    exit 1
else
    $ECHO_ "yes"
fi

## check genomeCoverageBed
$ECHO_N "Checking for genomeCoverageBed..."
if ! type $GENOMECOVERAGEBED > /dev/null 2>&1
then
    $ECHO_ "no"
    $ECHO_E "\tgenomeCoverageBed (bedtools) is required in \$PATH"
    exit 1
else
    $ECHO_ "yes"
fi

## check bamToBed
$ECHO_N "Checking for bamToBed..."
if ! type $BAMTOBED > /dev/null 2>&1
then
    $ECHO_ "no"
    $ECHO_E "\tbamToBed (bedtools) is required in \$PATH"
    exit 1
else
    $ECHO_ "yes"
fi

## check required scripts
$ECHO_N "Checking for addXS.pl... "
if [ ! -e "$SRC/perl/addXS.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: addXS.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for separatestrand.pl... "
if [ ! -e "$SRC/perl/separatestrand.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: separatestrand.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for rmwrongpairmap.pl... "
if [ ! -e "$SRC/perl/rmwrongpairmap.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: rmwrongpairmap.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for sortbedid.pl... "
if [ ! -e "$SRC/perl/sortbedid.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: sortbedid.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for joinpairbed12.pl... "
if [ ! -e "$SRC/perl/joinpairbed12.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: joinpairbed12.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for gettsc.pl... "
if [ ! -e "$SRC/perl/gettsc.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: gettsc.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for getisoforms.pl... "
if [ ! -e "$SRC/perl/getisoforms.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: getisoforms.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for separatetscexon.pl... "
if [ ! -e "$SRC/perl/separatetscexon.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: separatetscexon.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for fusionstrandtsc.pl... "
if [ ! -e "$SRC/perl/fusionstrandtsc.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: fusionstrandtsc.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for jointscexon.pl... "
if [ ! -e "$SRC/perl/jointscexon.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: jointscexon.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for fusionintron.pl... "
if [ ! -e "$SRC/perl/fusionintron.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: fusionintron.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for segment.pl... "
if [ ! -e "$SRC/perl/segment.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: segment.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for getsplicedtsc.pl... "
if [ ! -e "$SRC/perl/getsplicedtsc.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: getsplicedtsc.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for runtsccompare.pl... "
if [ ! -e "$SRC/perl/runtsccompare.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: runtsccompare.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for tsccomparechr.pl... "
if [ ! -e "$SRC/perl/tsccomparechr.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: tsccomparechr.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for unscomparechr.pl... "
if [ ! -e "$SRC/perl/unscomparechr.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: unscomparechr.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for getcommontsc.pl... "
if [ ! -e "$SRC/perl/getcommontsc.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: getcommontsc.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for posnegcomparechr.pl... "
if [ ! -e "$SRC/perl/posnegcomparechr.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: posnegcomparechr.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for extractintrons.pl... "
if [ ! -e "$SRC/perl/extractintrons.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: extractintrons.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi
$ECHO_N "Checking for getbasecovchr.pl... "
if [ ! -e "$SRC/perl/getbasecovchr.pl" ]
then
    $ECHO_ "no"
    $ECHO_E "\tError: getbasecovchr.pl is required in $SRC/perl"
    exit 1
else
    $ECHO_ "yes"
fi

## create header and genome file
random_bam=`head -1 $PREFIX/rnaseq/bamlist.txt`
$SAMTOOLS view -H $random_bam -o $PREFIX/rnaseq/header.sam;
grep "@SQ" $PREFIX/rnaseq/header.sam | awk '{for (i=1; i<=NF; i++) if ($i ~ /SN:/ || $i ~ /LN:/) printf $i "\t"; printf "\n"};' | sed 's/SN://g' | sed 's/LN://g' > $PREFIX/genome/genomesize.txt

## if not given MaxInsertSize, max given gene size is used
if [ ! "$MIS" ]
then
    if [ ! -s "$GFF" ]
    then
	MIS=1000
    else
	## compute max gene size
	awk '{if ($3=="gene") print $5-$4}' $GFF > $PREFIX/genome/genesize.txt
	maxgene=0
	while read line
	do
	    if [ $line -gt $maxgene ] 
	    then
		maxgene=$line
	    fi
	done < $PREFIX/genome/genesize.txt
	MIS=$maxgene
    fi
fi

while read bam
do
    instance=`basename $bam`
    instance=`basename $instance .bam`
    instance=`basename $instance accepted_hits`
    instance=`basename $instance .`
    instance=`basename $instance _`
    
    ## make init scripts
    perl $SRC/perl/makeInit.pl $CLUSTER_OPTION $PAIREND_OPTION $INDEX_OPTION $WHOLEDE_OPTION --library-type=$LIBTYPE --path=$PREFIX --bam=$bam --src=$SRC --output=$PREFIX/script/init_$instance.sh --GenomeSize=$PREFIX/genome/genomesize.txt --MaxInsertSize=$MIS --CPU=$MAXCPU --memory=$MEM
    
    ## make coverage scripts
    perl $SRC/perl/makeCoverage.pl $CLUSTER_OPTION $WHOLEDE_OPTION --path=$PREFIX --bam=$bam --src=$SRC --outputPrefix=$PREFIX/script/coverage_$instance --GenomeSize=$PREFIX/genome/genomesize.txt --library-type=$LIBTYPE --CPU=$MAXCPU --memory=$MEM
    
done < $PREFIX/rnaseq/bamlist.txt

## make GFFrefine script
perl $SRC/perl/makeGFFrefine.pl $CLUSTER_OPTION --path=$PREFIX --src=$SRC --gff=$GFF --output=$PREFIX/script/init_GFF.sh --feature=$FEATURE --GenomeSize=$PREFIX/genome/genomesize.txt --CPU=$MAXCPU --memory=$MEM

## make GFFprof scripts
MEMGFFPROF=`echo $MEM | sed 's#[GMK]##g'`
UNIT=`echo $MEM | sed 's#[0-9]##g'`
MEMGFFPROF=`expr $MEMGFFPROF \* $MAXCPU`
MEMGFFPROF=`expr $MEMGFFPROF \* 90`
MEMGFFPROF=`expr $MEMGFFPROF / 100`
perl $SRC/perl/makeGFFprof.pl $CLUSTER_OPTION $PAIREND_OPTION --path=$PREFIX --src=$SRC --gff=$GFF --output=$PREFIX/script/GFFprof --GenomeSize=$PREFIX/genome/genomesize.txt --feature=$FEATURE --replicate_intron=$REPLICATE_INTRON --replicate_exon=$REPLICATE_EXON --memory=$MEMGFFPROF$UNIT

chmod +x $PREFIX/script/*.sh
